@@ -1,8 +1,5 @@
 #pragma once
 
-#ifndef FF_INCLUDED_parsing_h
-#define FF_INCLUDED_parsing_h
-
 #include "fastfetch.h"
 
 #include <stdint.h>
@@ -22,6 +19,5 @@ void ffParseGTK(FFstrbuf* buffer, const FFstrbuf* gtk2, const FFstrbuf* gtk3, co
 void ffVersionToPretty(const FFVersion* version, FFstrbuf* pretty);
 int8_t ffVersionCompare(const FFVersion* version1, const FFVersion* version2);
 
-void ffParseSize(uint64_t bytes, FFBinaryPrefixType binaryPrefix, FFstrbuf* result);
-
-#endif
+void ffParseSize(uint64_t bytes, FFstrbuf* result);
+void ffParseTemperature(double celsius, FFstrbuf* buffer);
