@@ -1,23 +1,26 @@
 #include "media.h"
-#include "detection/internal.h"
 
-void ffDetectMediaImpl(const FFinstance* instance, FFMediaResult* media);
+void ffDetectMediaImpl(FFMediaResult* media);
 
-const FFMediaResult* ffDetectMedia(const FFinstance* instance)
+const FFMediaResult* ffDetectMedia(void)
 {
-    FF_DETECTION_INTERNAL_GUARD(FFMediaResult,
-        ffStrbufInitA(&result.error, 0);
-        ffStrbufInit(&result.busNameShort);
+    static FFMediaResult result;
+
+    if (result.error.chars == NULL)
+    {
+        ffStrbufInit(&result.error);
+        ffStrbufInit(&result.playerId);
         ffStrbufInit(&result.player);
         ffStrbufInit(&result.song);
         ffStrbufInit(&result.artist);
         ffStrbufInit(&result.album);
         ffStrbufInit(&result.url);
         ffStrbufInit(&result.status);
-
-        ffDetectMediaImpl(instance, &result);
+        ffDetectMediaImpl(&result);
 
         if(result.song.length == 0 && result.error.length == 0)
             ffStrbufAppendS(&result.error, "No media found");
-    );
+    }
+
+    return &result;
 }
