@@ -5,12 +5,13 @@
 #include "common/properties.h"
 #include "common/settings.h"
 #include "detection/os/os.h"
+#include "util/stringUtils.h"
 
 #include <dirent.h>
 
 static uint32_t getNumElementsImpl(const char* dirname, unsigned char type)
 {
-    DIR* dirp = opendir(dirname);
+    FF_AUTO_CLOSE_DIR DIR* dirp = opendir(dirname);
     if(dirp == NULL)
         return 0;
 
@@ -25,8 +26,6 @@ static uint32_t getNumElementsImpl(const char* dirname, unsigned char type)
     if(type == DT_DIR && num_elements >= 2)
         num_elements -= 2; // accounting for . and ..
 
-    closedir(dirp);
-
     return num_elements;
 }
 
@@ -41,26 +40,19 @@ static uint32_t getNumElements(FFstrbuf* baseDir, const char* dirname, unsigned
 
 static uint32_t getNumStringsImpl(const char* filename, const char* needle)
 {
-    FILE* file = fopen(filename, "r");
-    if(file == NULL)
+    FF_STRBUF_AUTO_DESTROY content = ffStrbufCreate();
+    if (!ffReadFileBuffer(filename, &content))
         return 0;
 
     uint32_t count = 0;
-
-    char* line = NULL;
-    size_t len = 0;
-
-    while(getline(&line, &len, file) != EOF)
+    char *iter = content.chars;
+    size_t needleLength = strlen(needle);
+    while ((iter = memmem(iter, content.length - (size_t)(iter - content.chars), needle, needleLength)) != NULL)
     {
-        if(strstr(line, needle) != NULL)
-            ++count;
+        ++count;
+        iter += needleLength;
     }
 
-    if(line != NULL)
-        free(line);
-
-    fclose(file);
-
     return count;
 }
 
@@ -73,11 +65,11 @@ static uint32_t getNumStrings(FFstrbuf* baseDir, const char* filename, const cha
     return num_elements;
 }
 
-static uint32_t getSQLite3Int(const FFinstance* instance, FFstrbuf* baseDir, const char* dbPath, const char* query)
+static uint32_t getSQLite3Int(FFstrbuf* baseDir, const char* dbPath, const char* query)
 {
     uint32_t baseDirLength = baseDir->length;
     ffStrbufAppendS(baseDir, dbPath);
-    uint32_t num_elements = (uint32_t) ffSettingsGetSQLite3Int(instance, baseDir->chars, query);
+    uint32_t num_elements = (uint32_t) ffSettingsGetSQLite3Int(baseDir->chars, query);
     ffStrbufSubstrBefore(baseDir, baseDirLength);
     return num_elements;
 }
@@ -132,12 +124,10 @@ static uint32_t getNixPackagesImpl(char* path)
     if(!ffPathExists(path, FF_PATHTYPE_DIRECTORY))
         return 0;
 
-    FFstrbuf output;
-    ffStrbufInitA(&output, 128);
+    FF_STRBUF_AUTO_DESTROY output = ffStrbufCreateA(128);
 
-    //https://github.com/LinusDierheimer/fastfetch/issues/195#issuecomment-1191748222
-    FFstrbuf command;
-    ffStrbufInitA(&command, 255);
+    //https://github.com/fastfetch-cli/fastfetch/issues/195#issuecomment-1191748222
+    FF_STRBUF_AUTO_DESTROY command = ffStrbufCreateA(255);
     ffStrbufAppendS(&command, "for x in $(nix-store --query --requisites ");
     ffStrbufAppendS(&command, path);
     ffStrbufAppendS(&command, "); do if [ -d $x ]; then echo $x ; fi ; done | cut -d- -f2- | egrep '([0-9]{1,}\\.)+[0-9]{1,}' | egrep -v '\\-doc$|\\-man$|\\-info$|\\-dev$|\\-bin$|^nixos-system-nixos-' | uniq | wc -l");
@@ -149,12 +139,7 @@ static uint32_t getNixPackagesImpl(char* path)
         NULL
     });
 
-    int result = (int) strtol(output.chars, NULL, 10);
-
-    ffStrbufDestroy(&command);
-    ffStrbufDestroy(&output);
-
-    return (uint32_t) result;
+    return (uint32_t) strtol(output.chars, NULL, 10);
 }
 
 static uint32_t getNixPackages(FFstrbuf* baseDir, const char* dirname)
@@ -177,7 +162,7 @@ static uint32_t getXBPSImpl(FFstrbuf* baseDir)
     struct dirent *entry;
     while((entry = readdir(dir)) != NULL)
     {
-        if(entry->d_type != DT_REG || strncasecmp(entry->d_name, "pkgdb-", 6) != 0)
+        if(entry->d_type != DT_REG || !ffStrStartsWithIgnCase(entry->d_name, "pkgdb-"))
             continue;
 
         ffStrbufAppendC(baseDir, '/');
@@ -203,6 +188,9 @@ static uint32_t getSnap(FFstrbuf* baseDir)
 {
     uint32_t result = getNumElements(baseDir, "/snap", DT_DIR);
 
+    if (result == 0)
+        result = getNumElements(baseDir, "/var/lib/snapd/snap", DT_DIR);
+
     //Accounting for the /snap/bin folder
     return result > 0 ? result - 1 : 0;
 }
@@ -227,9 +215,9 @@ static uint32_t getFlatpak(FFstrbuf* baseDir, const char* dirname)
 #include <rpm/rpmdb.h>
 #include <rpm/rpmlog.h>
 
-static uint32_t getRpmFromLibrpm(const FFinstance* instance)
+static uint32_t getRpmFromLibrpm(void)
 {
-    FF_LIBRARY_LOAD(rpm, &instance->config.librpm, 0, "librpm" FF_LIBRARY_EXTENSION, 12)
+    FF_LIBRARY_LOAD(rpm, &instance.config.library.librpm, 0, "librpm" FF_LIBRARY_EXTENSION, 12)
     FF_LIBRARY_LOAD_SYMBOL(rpm, rpmReadConfigFiles, 0)
     FF_LIBRARY_LOAD_SYMBOL(rpm, rpmtsCreate, 0)
     FF_LIBRARY_LOAD_SYMBOL(rpm, rpmtsInitIterator, 0)
@@ -265,36 +253,38 @@ static uint32_t getRpmFromLibrpm(const FFinstance* instance)
 
 #endif //FF_HAVE_RPM
 
-static void getPackageCounts(const FFinstance* instance, FFstrbuf* baseDir, FFPackagesResult* packageCounts)
+static void getPackageCounts(FFstrbuf* baseDir, FFPackagesResult* packageCounts)
 {
     packageCounts->apk += getNumStrings(baseDir, "/lib/apk/db/installed", "C:Q");
-    packageCounts->dpkg += getNumStrings(baseDir, "/var/lib/dpkg/status", "Status: ");
+    packageCounts->dpkg += getNumStrings(baseDir, "/var/lib/dpkg/status", "Status: install ok installed");
     packageCounts->emerge += countFilesRecursive(baseDir, "/var/db/pkg", "SIZE");
     packageCounts->eopkg += getNumElements(baseDir, "/var/lib/eopkg/package", DT_DIR);
     packageCounts->flatpakSystem += getFlatpak(baseDir, "/var/lib/flatpak");
     packageCounts->nixDefault += getNixPackages(baseDir, "/nix/var/nix/profiles/default");
     packageCounts->nixSystem += getNixPackages(baseDir, "/run/current-system");
     packageCounts->pacman += getNumElements(baseDir, "/var/lib/pacman/local", DT_DIR);
-    packageCounts->pkg += getSQLite3Int(instance, baseDir, "/var/db/pkg/local.sqlite", "SELECT count(id) FROM packages");
-    packageCounts->rpm += getSQLite3Int(instance, baseDir, "/var/lib/rpm/rpmdb.sqlite", "SELECT count(blob) FROM Packages");
+    packageCounts->pkgtool += getNumElements(baseDir, "/var/log/packages", DT_REG);
+    packageCounts->rpm += getSQLite3Int(baseDir, "/var/lib/rpm/rpmdb.sqlite", "SELECT count(*) FROM Packages");
     packageCounts->snap += getSnap(baseDir);
     packageCounts->xbps += getXBPS(baseDir, "/var/db/xbps");
     packageCounts->brewCask += getNumElements(baseDir, "/home/linuxbrew/.linuxbrew/Caskroom", DT_DIR);
     packageCounts->brew += getNumElements(baseDir, "/home/linuxbrew/.linuxbrew/Cellar", DT_DIR);
+    packageCounts->paludis += countFilesRecursive(baseDir, "/var/db/paludis/repositories", "environment.bz2");
+    packageCounts->opkg += getNumStrings(baseDir, "/usr/lib/opkg/status", "Package:"); // openwrt
 }
 
-static void getPackageCountsRegular(const FFinstance* instance, FFstrbuf* baseDir, FFPackagesResult* packageCounts)
+static void getPackageCountsRegular(FFstrbuf* baseDir, FFPackagesResult* packageCounts)
 {
-    getPackageCounts(instance, baseDir, packageCounts);
+    getPackageCounts(baseDir, packageCounts);
 
     uint32_t baseDirLength = baseDir->length;
-    ffStrbufAppendS(baseDir, FASTFETCH_TARGET_DIR_ETC"/pacman-mirrors.conf");
+    ffStrbufAppendS(baseDir, FASTFETCH_TARGET_DIR_ETC "/pacman-mirrors.conf");
     if(ffParsePropFile(baseDir->chars, "Branch =", &packageCounts->pacmanBranch) && packageCounts->pacmanBranch.length == 0)
         ffStrbufAppendS(&packageCounts->pacmanBranch, "stable");
     ffStrbufSubstrBefore(baseDir, baseDirLength);
 }
 
-static void getPackageCountsBedrock(const FFinstance* instance, FFstrbuf* baseDir, FFPackagesResult* packageCounts)
+static void getPackageCountsBedrock(FFstrbuf* baseDir, FFPackagesResult* packageCounts)
 {
     uint32_t baseDirLength = baseDir->length;
 
@@ -315,11 +305,11 @@ static void getPackageCountsBedrock(const FFinstance* instance, FFstrbuf* baseDi
     {
         if(entry->d_type != DT_DIR)
             continue;
-        if(strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
+        if(ffStrEquals(entry->d_name, ".") || ffStrEquals(entry->d_name, ".."))
             continue;
 
         ffStrbufAppendS(baseDir, entry->d_name);
-        getPackageCounts(instance, baseDir, packageCounts);
+        getPackageCounts(baseDir, packageCounts);
         ffStrbufSubstrBefore(baseDir, baseDirLength2);
     }
 
@@ -327,26 +317,25 @@ static void getPackageCountsBedrock(const FFinstance* instance, FFstrbuf* baseDi
     ffStrbufSubstrBefore(baseDir, baseDirLength);
 }
 
-void ffDetectPackagesImpl(const FFinstance* instance, FFPackagesResult* result)
+void ffDetectPackagesImpl(FFPackagesResult* result, FF_MAYBE_UNUSED FFPackagesOptions* options)
 {
-    FF_STRBUF_AUTO_DESTROY baseDir;
-    ffStrbufInitA(&baseDir, 512);
+    FF_STRBUF_AUTO_DESTROY baseDir = ffStrbufCreateA(512);
     ffStrbufAppendS(&baseDir, FASTFETCH_TARGET_DIR_ROOT);
 
-    if(ffStrbufIgnCaseEqualS(&ffDetectOS(instance)->id, "bedrock"))
-        getPackageCountsBedrock(instance, &baseDir, result);
+    if(ffStrbufIgnCaseEqualS(&ffDetectOS()->id, "bedrock"))
+        getPackageCountsBedrock(&baseDir, result);
     else
-        getPackageCountsRegular(instance, &baseDir, result);
+        getPackageCountsRegular(&baseDir, result);
 
     // If SQL failed, we can still try with librpm.
     // This is needed on openSUSE, which seems to use a proprietary database file
     // This method doesn't work on bedrock, so we do it here.
     #ifdef FF_HAVE_RPM
         if(result->rpm == 0)
-            result->rpm = getRpmFromLibrpm(instance);
+            result->rpm = getRpmFromLibrpm();
     #endif
 
-    ffStrbufSet(&baseDir, &instance->state.platform.homeDir);
+    ffStrbufSet(&baseDir, &instance.state.platform.homeDir);
     result->nixUser = getNixPackages(&baseDir, "/.nix-profile");
     result->flatpakUser = getFlatpak(&baseDir, "/.local/share/flatpak");
 }
